<?php

namespace App;

use App\Models\City;
use App\Models\Comment;
use App\Models\CountryCodes;
use App\Models\Message;
use App\Models\Notification;
use App\Models\Product;
use App\Models\Room;
use App\Models\Spam;
use App\Models\Subscription;
use App\Models\Transfer;
use App\Models\UserRate;
use App\Models\UserToken;
use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Tymon\JWTAuth\Contracts\JWTSubject;

class User extends Authenticatable implements JWTSubject


{
    use Notifiable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'name',
        'email',
        'password',
        'mobile',
        'image',
        'online',
        'type',
        'verified',
        'code',
        'status', 'lat',
        'lng',
        'city_id',
        'country_code_id',
        'vip',
        'verified_code',
        'end_subscription'

    ];
    protected $appends = ['total_rates', 'total_transfers'];

    public function getTotalTransfersAttribute()
    {
        $amount = Transfer::where('user_id', $this->id)->where('approved', 'true')->sum('amount');
        return $amount;
    }

    public function getTotalRatesAttribute()
    {
        $count = UserRate::where(['user_id' => $this->id, 'type' => 'good'])->count();
        return $count;
    }

    public function setImageAttribute()
    {
        $file = request()->file('image');
        $destinationPath = 'images/users/';
        $filename = time() . '.' . $file->getClientOriginalExtension();
        $file->move($destinationPath, $filename);
        $this->attributes['image'] = $filename;
    }

//    public function getImageAttribute()
//    {
//        if ($this->attributes['image']) {
//            return asset('images/users/') . '/' . $this->attributes['image'];
//        }
//        return asset('images/users/default.jpg');
//    }

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'password', 'remember_token',
    ];

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];
    protected $table = 'users';

    public function getJWTIdentifier()
    {
        return $this->getKey();
    }

    public function getJWTCustomClaims()
    {
        return [];
    }

    // relations

    public function updateLocation($lat, $lng)
    {
        User::whereId($this->id)->update([
            'lat' => $lat,
            'lng' => $lng
        ]);
    }

    public function sender()
    {
        return $this->hasMany(Room::class, 'sender_id');
    }

    public function receiver()
    {
        return $this->hasMany(Room::class, 'receiver_id');
    }

    public function tokens()
    {
        return $this->hasMany(UserToken::class, 'user_id');
    }

    public function product()
    {
        return $this->hasMany(Product::class, 'user_id');
    }

    public function followers()
    {
        return $this->belongsToMany(User::class, 'followers', 'follower_id', 'user_id');
    }

    public function followings()
    {
        return $this->belongsToMany(User::class, 'followers', 'user_id', 'follower_id');
    }

    public function rates()
    {
        return $this->belongsToMany(User::class, 'user_rates', 'rated_id', 'user_id');
    }

    public function followed_products()
    {
        return $this->belongsToMany(Product::class, 'followers', 'user_id', 'product_id');
    }

    public function favourite_products()
    {
        return $this->belongsToMany(Product::class, 'favourites', 'user_id', 'product_id');
    }

    public function notifications()
    {
        return $this->hasMany(Notification::class, 'user_id');
    }

    public function subscriptions()
    {
        return $this->hasMany(Subscription::class, 'user_id');
    }

    public function spams()
    {
        return $this->hasMany(Spam::class, 'user_id');
    }

    public function comments()
    {
        return $this->hasMany(Comment::class, 'user_id');
    }

    public function send_messages()
    {
        return $this->hasMany(Message::class, 'sender_id');
    }

    public function receive_messages()
    {
        return $this->hasMany(Message::class, 'receiver_id');
    }

    public function city()
    {
        return $this->belongsTo(City::class, 'city_id');
    }

    public function country_code()
    {
        return $this->belongsTo(CountryCodes::class, 'country_code_id');
    }

    public function views_products()
    {
        return $this->belongsToMany(Product::class, 'views', 'user_id', 'product_id');
    }
}
